# Copyright (c) 2010,2011 Linaro
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abrek.testdef

INSTALLSTEPS = ['git clone git://github.com/janimo/firefox-startup-timing.git']
DEPS = ['firefox', 'git-core', 'gcalctool']
RUNSTEPS = ['cd firefox-startup-timing; ./firefox_startup_timing.sh']
PATTERN = "^(?P<test_case_id>\w+):(?P<measurement>\d+)"

firefoxinst = abrek.testdef.AbrekTestInstaller(INSTALLSTEPS, deps=DEPS)
firefoxrun = abrek.testdef.AbrekTestRunner(RUNSTEPS)
firefoxparser = abrek.testdef.AbrekTestParser(PATTERN,
               appendall={'units':'ms', 'result':'pass'})
testobj = abrek.testdef.AbrekTest(testname="firefox", installer=firefoxinst,
                                  runner=firefoxrun, parser=firefoxparser)
